/*******************************************************************************
* Copyright (C) 2005, 2006 Novell, Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Novell, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*
* Author: Jon Carey (jcarey@novell.com)
******************************************************************************/
#ifndef CMPIUTIL_EXEC_H_GUARD__
#define CMPIUTIL_EXEC_H_GUARD__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdlib.h>

/**
 * Execute a command.
 * The command will inherit stdin, stdout, and stderr from the parent
 * process.  This function will not search the path for argv[0], so
 * the absolute path to the binary should be specified.  If the path needs
 * to be searched, you can set argv[0] = "/bin/sh"; argv[1] = "-c";
 * and then fill in the rest of the array with the command you wish to
 * execute.
 * This function blocks until the child process exits.  Be careful that
 * the command you run doesn't hang.  It is recommended to use
 * cmpiutilExecuteProcessAndGatherOutputAndError if the command is untrusted.
 *
 * @param argv A NULL terminated cha* array that specifies the command to run.
 *  argv[0] is the binary to be executed.
 *  argv[1]..argv[n-1] are the command line parameters to the command.
 *  argv[n] NULL value to terminate array
 *
 * @param envp an array of strings  of the form "key=value", which are passed
 *  as environment to the new program. envp must be terminated by a null
 *  pointer. envp may be NULL, in which case the current process's environment
 *  variables will be used.
 *
 * @return 127 if the execve() call for command[0] fails, -1 if there was
 * another error and the return code of the command otherwise.
 */
int
cmpiutilSafeSystem(
	char* const argv[],
	char* const envp[]);

/**
 * Execute a command.
 * The command's stdin, stdout, and stderr will be connected via pipes to
 * the parent process that can be accessed from from the file handles
 * that are set on return.
 * This function will not search the path for argv[0], so
 * the absolute path to the binary should be specified. If the path needs
 * to be searched, you can set argv[0] = "/bin/sh"; argv[1] = "-c";
 * and then fill in the rest of the array with the command you wish to
 * execute.
 * This function does *not* block until the child process exits.
 * If the binary specified in argv[0] does not exist, execv() will fail,
 * and the return code of the sub-process will be 127. However, this is not
 * distinguishable from the command process returning 127.
 *
 * @param argv A NULL terminated char* array that specifies the command to run.
 *  argv[0] is the binary to be executed.
 *  argv[1]..argv[n-1] are the command line parameters to the command.
 *  argv[n] A NULL value terminating the array
 *
 * @param envp an array of strings  of the form "key=value", which are passed
 *  as environment to the new program. envp must be terminated by a null
 *  pointer. envp may be 0, in which case the current process's environment
 *  variables will be used.
 *
 * @param childsStdin A pointer to an int that will hold a file handle to the
 *  child's stdin on successful return.
 *
 * @param childsStdout A pointer to an int that will hold a file handle to the
 *  child's stdout on successful return.
 *
 * @param childsStderr A pointer to an int that will hold a file handle to the
 *  child's stderr on successful return.
 *
 * @return The childs process ID on success. Otherwise -1 on error and errno
 * can be inspected for further information.
 * On success the childsStdIn, childsStdout and childsStderr parameters will
 * be set with the child process stdin, stdout and stderr respectively. After
 * the caller has finished processing the results of the child process, it is
 * responsible for closing these file handles and doing a waitpid(...) on the
 * child's process ID to free up the child process resources. See the man page
 * on waitpid(...).
 */
int cmpiutilSafePopen(
	char* const argv[],
	char* const envp[],
	int* childsStdin,
	int* childsStdout,
	int* childsStderr);

/**
 * Run a process, collect the output, and wait for it to exit.  The function
 * returns when the process exits. In the case that the child process doesn't
 * exit, if a timout is specified, then -1 is returned and errno is set to
 * ETIMEDOUT. If the child process outputs more bytes than what is specified
 * in the outputLimit(child's stdout) or errOutputLimit(child's stderr),
 * -1 will be returned with errno set to EOVERFLOW.
 * This function will not search the path for argv[0], so the absolute path to
 * the binary should be specified.  If the path needs to be searched
 * to be searched, you can set argv[0] = "/bin/sh"; argv[1] = "-c";
 * and then fill in the rest of the array with the command you wish to
 * execute. Exercise caution when doing this, as you may be creating a
 * security hole.
 * If the process does not terminate by itself, or if an error is
 * encountered because a limit has been reached (time or output), then the
 * the following steps will be taken to attempt to terminate the child
 * process.
 * 1. The input and output pipes will be closed.  This may cause the
 *    child to get a SIGPIPE which may terminate it.
 * 2. If the child still hasn't terminated after 10 seconds, a SIGTERM
 *    is sent.
 * 3. If the child still hasn't terminated after 10 seconds, a SIGKILL
 *    is sent.			
 *
 * @param argv A NULL terminated cha* array that specifies the command to run.
 *  argv[0] is the binary to be executed.
 *  argv[1]..argv[n-1] are the command line parameters to the command.
 *  argv[n] NULL value to terminate array
 *
 * @param envp an array of strings  of the form "key=value", which are passed
 *  as environment to the new program. envp must be terminated by a null
 *  pointer. envp may be NULL, in which case the current process's environment
 *  variables will be used.
 *
 * @param output REQUIRED OUT parameter. This points to a string pointer that
 *  will be set to point to a buffer that will contain the stdout output from
 *  the child process. If this pointer is set on return, it MUST be freed or
 *  you will have a memory leak.
 *
 * @param outputSize REQUIRED OUT parameter. On entry this points to a size_t.
 *  This size_t will be set to the number of characters that have 
 *  been written to the buffer pointed to by the output parameter
 *  (child's stdout).
 *
 * @param outputLimit Specifies the maximum number of characters of the
 *  childs stdout that should be captured before an error (EOVERFLOW) is
 *  generated. If this is exceeded the client will be terminated. If this
 *  value is specified as -1 then outputLimit is considered unlimited.
 *
 * @param errOutput OPTIONAL OUT parameter. This points to a string pointer that
 *  will be set to point to a buffer that will contain the stderr output from
 *  the child process. If this pointer is set on return, it MUST be freed or
 *  you will have a memory leak. If this parameter is NULL, the child's stderr
 *  output will not be captured.
 *
 * @param errOutputSize OPTIONAL OUT parameter. On entry this should point to a
 *  size_t if the errOutput parameter is specified. This size_t will
 *  be set to the number of characters that have been written to the buffer
 *  pointed to by the errOutput parameter (child's stderr).
 *
 * @param errOutputLimit Specifies the maximum number of characters of the
 *  childs stderr that should be captured before an error (EOVERFLOW) is
 *  generated. If this is exceeded the client will be terminated. If this
 *  value is specified as -1 then errOutputLimit is considered unlimited.
 *
 * @param input Data to be written to the child's stdin. After the data has been
 *  written, stdin will be closed.
 *
 * @param processstatus An out parameter, which will contain the process
 *  status.  It is only valid if the funtion returns without error. In the
 *  case an error is returned (-1), it is undefined. It should be evaluated
 *  using the family of macros (WIFEXITED(), WEXITSTATUS(), etc.) from
 *  "sys/wait.h"
 *
 * @param timeOutsecs Specifies the maximum number of seconds to wait for a
 *  response from the child when reading stdout/stderr or waiting for
 *  the termination after all of the stdout/stderr has been read. If this
 *  timeout is execeed during the operation the client will be terminated
 *  and an error will be returned (-1) with errno set to ETIMEDOUT.
 *
 * @return 0 on success. Otherwise -1 with errno set to indicate the problem
 *  that was encountered.
 */
int
cmpiutilExecuteProcessAndGatherOutputAndError(
	char* const argv[],
	char* const envp[],
	char** output,
	size_t* outputSize,
	int outputLimit,
	char** errOutput,
	size_t* errOutputSize,
	int errOutputLimit,
	const char* input,
	int* processStatus,
	int timeOutSecs);

#ifdef __cplusplus
}
#endif

#endif	/* CMPIUTIL_EXEC_H_GUARD__ */
