/*******************************************************************************
* Copyright (C) 2005, 2006 Novell, Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Novell, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*
* Author: Jon Carey (jcarey@novell.com)
******************************************************************************/
#ifndef CMPIUTIL_IOUTILS_H_GUARD__
#define CMPIUTIL_IOUTILS_H_GUARD__

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The cmpiutilReadLine function will read a line from a given file stream. It
 * allocates enough memory to hold the longest line it encounters in the
 * file. The address of the memory it allocates is stored in a pointer
 * provided by the caller. The caller is responsible for deallocating this
 * memory when processing of the file completes. On the initial call to
 * cmpiutilReadLine the bfr parameter should be set to NULL with the bfrlen
 * pointer pointer pointing to an unsigned int with a value of zero.
 * On subsequent calls to this function the same pointers should be 
 * passed by the caller. Below is an example of its usage:
 *
 *	FILE* fd;
 *	size_t bfrLen = 0;
 *	char *p, *bufPtr = NULL;
 *
 *	fd = fopen("mytextfile.txt", "r");
 *	for(p = cmpiutilReadLine(fd, &bufPtr, &bfrLen); p;
 *			p = cmpiutilReadLine(fd, &bufPtr, &bfrLen)) {
 *		printf("Line: %s\n", bufPtr);
 *	}
 *	fclose(fd);
 *	if (bufPtr)
 *		free(bufPtr);
 *
 * @param fd A handle to an open file to read the next line from.
 * @param bfr A pointer to a char pointer that the address of the
 * 		allocated memory will be stored. The caller is expected to
 * 		free this memory after it has completed processing of the
 * 		file.
 * @param bfrLen This is a pointer to a size_t that cmpiutilReadLine
 * 		will store the amount of memory it has allocated.
 * @return A NULL pointer if EOF or error has been encountered. Otherwise
 * 		a pointer to the beginning of the line it has just read.
 */
char* cmpiutilReadLine(FILE* fd, char **bfr, size_t* const bfrLen);

/**
 * Create a temp file in the standard temp file location for the platform.
 * The file will be open in read/write mode. This file will optionally be
 * deleted automatically when it is closed, depending on whether or not
 * the user specified the fileNameOut parameter.
 * @param fileNameOut Pointer to a buffer that will contain the name of the
 * 		temp file on successful return. If this param is specified as NULL,
 * 		then the file will automatically be deleted when it is closed.
 * @param fileNameOutLen The length of the buffer pointed to by the
 * 		fileNameOut parameter.
 * @return A file handle to a temp file. This file will be open in
 * 		read/write mode and will be automatically deleted if the caller
 * 		passed in a NULL for the fileNameOut parameter. NULL will be
 * 		returned if an error is encountered trying to perform the
 * 		operation. If this occurs, it means that cmpiutilCreateTempFile
 * 		was unable to create the temp file (mkstemp) or unable
 * 		to create a stream handle from the handle returned from
 * 		mkstemp (fdopen). In this case errno can be inspected
 * 		for clues.
 */
FILE* cmpiutilCreateTempFile(char* fileNameOut, size_t fileNameOutLen);


/**
 * Create a temp file in a given directory.
 * The file will be open in read/write mode. This file will optionally be
 * deleted automatically when it is closed, depending on whether or not
 * the user specified the fileNameOut parameter.
 *
 * @param dirName The name of the directory to place the file. If this
 *  paramerter is specified as NULL, then the behavior is identical to
 *  cmpiutilCreateTempFile.
 *
 * @param fileNameOut Pointer to a buffer that will contain the name of the
 * 	temp file on successful return. If this param is specified as NULL,
 * 	then the file will automatically be deleted when it is closed.
 *
 * @param fileNameOutLen The length of the buffer pointed to by the
 * 	fileNameOut parameter.
 *
 * @return A file handle to a temp file. This file will be open in
 * 	read/write mode and will be automatically deleted if the caller
 * 	passed in a NULL for the fileNameOut parameter. NULL will be
 * 	returned if an error is encountered trying to perform the
 * 	operation. If this occurs, it means that cmpiutilCreateTempFile
 * 	was unable to create the temp file (mkstemp) or unable
 * 	to create a stream handle from the handle returned from
 * 	mkstemp (fdopen). In this case errno can be inspected
 * 	for clues.
 */
FILE* cmpiutilCreateTempFileInDir(const char* dirName, char* fileNameOut,
	size_t fileNameOutLen);


#ifdef __cplusplus
}
#endif

#endif	/* CMPIUTIL_IOUTILS_H_GUARD__ */
