/*******************************************************************************
* Copyright (C) 2005, 2006 Novell, Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Novell, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*
* Author: Jon Carey (jcarey@novell.com)
******************************************************************************/
#ifndef FILEMODIFIER_H_GUARD__
#define FILEMODIFIER_H_GUARD__

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The cmpiutilFileModifier_CBS structure encapsulates all the callback functions
 * that can be called during the file modification performed by the
 * FileModifier_ModifyFile function. If any of the function pointers are
 * NULL in the cmpiutilFileModifier_CBS structure that is passed to the
 * FileModifier_ModifyFile function, then they are ignored. FYI: If all
 * the function pointers are NULL, then the modification will not take
 * place. The only thing that would happen in this case is the file
 * will not be modified.
 */
typedef struct
{
	/**
	 * Called right after the file modifier opend the temp file for a modify
	 * operation. The file filter has not written anything to the file yet.
	 * @parm ostrm The file handle for the tempfile that was opened for
	 * 		modification. This is an output only handle.
	 * @return 1 if the callback wrote anything. Otherwise 0.
	 */
	int (*fileOpened)(FILE* const ostrm);

	/**
	 * Called right before the file modifier closes the temp file for a modify
	 * operation. This is an output only handle.
	 * @parm ostrm The file handle for the tempfile that was opened for
	 * 		modification. This is an output only handle.
	 * @return 1 if the callback wrote anything to the file handle. Otherwise
	 * 		return 0.
	 */
	int (*fileClosing)(FILE* const ostrm);

	/**
	 * Called by the modifier to write a line to the temp file. The call back
	 * has the option of not writing the line to the file, writing an entirely
	 * different line(s) to the file, writing the given line to the file, or
	 * returning a -1 to abort the entire modification process.
	 * @param A pointer to a string that contains a line read from the original
	 * 		file. If not modifications are required, the callback should write
	 * 		this string to the given file handle (ostrm) followed by a new line.
	 * @param ostrm A file handle to the temp file that will contain the
	 * 		modified version of the file specified for modification.
	 * @param errMsgBfr This is a buffer that the callback place some
	 * 		descriptive text about an error that has been encountered. The
	 * 		callback _must_ not put data in this buffer that exceeds
	 * 		the length of this buffer which is specified by the errMsgBfrLen
	 * 		parameter. This text is ignored if the return value is >= 0.
	 * @param errMsgBfrLen The length of the buffer pointed to by the
	 * 		errMsgBfr parameter.
	 * @return	0: The line parameter was written as is.
	 * 			>0: A modified line was written to the file or the
	 * 				line was ignored.
	 * 			-1: An error occurred trying to write to the file.
	 * 			-2: Abort the modification process now.
	 */
	int (*writeLine)(const char* line, FILE* const ostrm, char* errMsgBfr,
		size_t errMsgBfrLen);

	/**
	 * This is called as a final step in the modifcation process. It
	 * is meant to give the the callback an opportunity to clean up any
	 * resources that may have been acquired during the modification process.
	 */
	void (*operationComplete)(void);

} cmpiutilFileModifier_CBS;

/**
 * The cmpiutilModifyFile function is used to perform custom modifications
 * to a text file. The intended use is the modification of system/application
 * configuration files. If the function completes successfully and has done
 * modifications, the original file will be found in the directory of the 
 * modified file with the name of <original file name>.orig
 * @param fileName The name of the file to be modified.
 * @param cbs A pointer to a cmpiutilFileModifier_CBS structure that contains the
 * 		callback function pointers that will be called during the
 * 		modification process.
 * @param errMsgBfr A buffer that will contain some descriptive text about
 *		an encountered error durring the process. If no error occurs, this
 *		parameter will be ignored. A NULL value for this parameter is
 *		acceptable.
 * @param errMsgBfrLen This specifies the length of the errMsgBfr if that
 * 		parameter was specified.
 * @return 0 On success. Otherwise a -1 if an error occurred.
 */
int cmpiutilModifyFile(
	const char* fileName, 
	const cmpiutilFileModifier_CBS* const cbs,
	char* errMsgBfr,
	size_t errMsgBfrLen);

/**
 * The cmpiutilModifyRollBack function can be used to roll back any changes
 * that were performed by the FileModifier_ModifyFile function. All it does
 * is move the <fileName>.orig file back to <fileName>. The intention is that
 * this would be done immediately following a FileModifier_Modify operation.
 * It maybe that other operations failed after a successful call to 
 * FileModifier_ModifyFile and the changes now need to be reversed before an
 * error is reported on the overall process.
 * @param fileName The name of the file that the original should be restored
 * 		for. The <fileName>.orig file must still exist.
 * @param errMsgBfr A buffer that will contain some descriptive text about
 *		an encountered error durring the process. If no error occurs, this
 *		parameter will be ignored. A NULL value for this parameter is
 *		acceptable.
 * @param errMsgBfrLen This specifies the length of the errMsgBfr if that
 * 		parameter was specified.
 * @return 0 if the roll back was successful. Otherwise -1.
 */
int cmpiutilModifyRollBack(
	const char* fileName,
	char* errMsgBfr,
	size_t errMsgBfrLen);

#ifdef __cplusplus
}
#endif

#endif	/* FILEMODIFIER_H_GUARD__ */
