/*******************************************************************************
* Copyright (C) 2005, 2006 Novell, Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Novell, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*
* Author: Jon Carey (jcarey@novell.com)
******************************************************************************/
#ifndef CMPIUTIL_STRING_H_GUARD__
#define CMPIUTIL_STRING_H_GUARD__
#include <string.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Determine if string ends with a given substring.
 * @param src The string to check the end for the given substring.
 * @param arg The substring the end of src is being check for.
 * @return If the src parameter does end with arg, then the address of
 * 	of the substring within src is returned. Otherwise NULL.
 * errno: EINVAL If invalid parameters are given.
 */
const char* cmpiutilStrEndsWith(const char* src, const char* arg);

/**
 * Determine if string starts with a given substring.
 * @param src The string to check the beginning for the given substring.
 * @param arg The substring the beginning of src is being check for.
 * @return If the src parameter does start with arg, then the address of
 * 	src is returned. Otherwise NULL.
 * errno: EINVAL If invalid parameters are given.
 */
const char* cmpiutilStrStartsWith(const char* src, const char* arg);

/**
 * Copy one string to another for a given length. This function is
 * similar to strncpy except it guarantees the result (dest) will
 * be NULL terminated.
 * @param dest The address to copy the string to.
 * @param src The source of the data to copy into the location pointed
 *  to by the dest parameter.
 * @param bytesToCopy This specifies the maximum number of bytes that
 *  can be copied to dest. This includes the NULL termination.
 * @return The results of the copy operation (dest)
 * errno: EINVAL If invalid parameters are given.
 */
const char* cmpiutilStrNCpy(char* dest, const char* src, size_t bytesToCopy);

/**
 * Append a string to the end of another string honoring a specified
 * maximum result len.
 * @param dest Address of the string to append to.
 * @param src Address of the string to append to dest.
 * @param maxResultLen The maximum length of the dest parameter on return.
 * 	cmpiutilStrAppend will not exceed this limit on the src buffer.
 * @return The address of the result string (dest) on success. NULL if an
 * 	error was encountered.
 * errno: EINVAL If invalid parameters are given.
 */
const char* cmpiutilStrAppend(char* dest, const char* src, size_t maxResultLen);

/**
 * Trip all leading white space from a string.
 * @param src The string to trim leading whitespace from.
 * @return A pointer to the trimmed string (src)
 * errno: N/A
 */
const char* cmpiutilStrLTrim(char* src);

/**
 * Trim all trailing space from a string.
 * @param src The string that the trailing space should be removed from.
 * @return A pointer to src with trailing space removed (src).
 * errno: N/A
 */
const char* cmpiutilStrRTrim(char* src);

/**
 * Trim all leading and trailing whitespace from a string.
 * @param src The string to trim all leading and trailing whitespace from.
 * @return A pointer to the trimmed string (src);
 * errno: N/A
 */
const char* cmpiutilStrTrim(char* src);

/**
 * Generated a simple hash value for the given string.
 * @parameter str The string to generate the hash value for.
 * @return The hash value for the given string.
 * errno: N/A
 */
uint32_t cmpiutilStrHash(const unsigned char* str);

/**
 * Determine the number of tokens there are in a string using a given set
 * of delimeters.
 * @param src The address of the string to count the token for.
 * @param delims The delimeters to use when counting the tokens. If this
 * 	parameter is NULL, then cmpiutilStrTokenCount will use the follwing
 * 	white space characters as the delimeters: " \f\n\r\t\v".
 * @return The number of tokens found in src
 * errno: EINVAL If invalid parameters are given.
 */
size_t cmpiutilStrTokenCount(const char* src, const char* delims);

/**
 * Tokenize a given string using a given set of delimeters. The result of
 * the function will be an argv style array (char* argv[]) where the last
 * string in the array is NULL. The caller is responsible for deallocating
 * the pointer returned from the function. All memory allocated will be done
 * in a single block (single malloc), so a single free will be all that is
 * required. Following is a code snippet of the usage of this function:
 *
 * 	const char *str = "This:;@will;return:::5:;@tokens;;;@@;@;:";
 *	char **strs = cmpiutilStrTokenize(str, ":;@");
 *	if (!strs) {
 *		// This could be there were tokens or an error
 *		// occurred during processing. Check errno
 *	}
 *	else {
 *		// Print out tokens and free the argv style arrary that was returned
 *		int i;
 *		for(i = 0; strs[i]; i++) {
 *			printf("\tstrs[%d] = %s\n", i, strs[i]);
 *		}
 *		free(strs);
 *	}
 *
 * @param src The string to tokenize
 * @param delims The delimeters to use to tokenize src.
 * @param numberOfElement If this pointer is not NULL, then the number of the
 *  elements in the return value will be stored here.
 * @return On success, an argv style array containing the tokens or NULL if
 * 	no tokens were found in src. A return value of NULL can be the
 * 	indication of an error. The only way to determine this is to check
 * 	the value of errno when a NULL value is returned.
 * errno:	EINVAL If invalid parameters are given.
 *			ENOMEM A memory allocation error was encountered during processing.
 */
char** cmpiutilStrTokenize(const char* src, const char* delims, size_t* numberOfElements);

/**
 * cmpiutilStrBuf represent a dynamic string buffer. It is created/destroyed/manipulted
 * by the cmpiutilStrBuf_xxx set of functions that follow.
 */
typedef struct {
    size_t buf_len;
    size_t str_len;
    char* bfr;
} cmpiutilStrBuf;

/**
 * Create an initialized cmpiutilStrBuf object.
 * @return A pointer to an initialized cmpiutilStrBuf object. This pointer must
 *  eventually be destroyed by calling cmpiutilStrBuf_destroy. This function will
 *  return NULL on failure.
 */
cmpiutilStrBuf* cmpiutilStrBuf_create();

/**
 * Destroy an cmpiutilStrBuf object and release the allocated memory associated with it.
 * @param psbuf A pointer to the cmpiutilStrBuf object to be destroyed.
 * @return 0 for success. Otherwise -1
 */
int cmpiutilStrBuf_destroy(cmpiutilStrBuf* psbuf);

/**
 * Append a string to the string represented by a given cmpiutilStrBuf object.
 * @param psbuf The cmpiutilStrBuf object to append the string to.
 * @param str A the string to append to the given cmpiutilStrBuf object.
 * @param n The number of character of str to append to the given
 *  cmpiutilStrBuf object. If this parameter is 0, then all of str will
 *  be appended.
 * @return A pointer to the modified string or NULL on failure.
 */
const char* cmpiutilStrBuf_cat(cmpiutilStrBuf* psbuf, const char* str, size_t n);

/**
 * Set the cmpiutilStrBuf object to a given string.
 * @param psbuf The cmpiutilStrBuf object to set.
 * @param str A the string to set the given cmpiutilStrBuf object to.
 * @param n The number of character of str to set the given
 *  cmpiutilStrBuf object to. If this parameter is 0, then all of str will
 *  be used.
 * @return A pointer to the string represented by the cmpiutilStrBuf object,
 *  or NULL on failure.
 */
const char* cmpiutilStrBuf_set(cmpiutilStrBuf* psbuf, const char* str, size_t n);

/**
 * Reset a given cmpiutilStrBuf object to an empty string.
 * @param psbuf The cmpiutilStrBuf object to reset.
 * @return 0 on success. Otherwise -1.
 */
int cmpiutilStrBuf_reset(cmpiutilStrBuf* psbuf);

/**
 * cmpiutilStrArray represent a dynamic string array. It is created/destroyed/manipulated
 * by the cmpiutilStrArray_xxx set of functions that follow.
 */
typedef struct {
    size_t num_strings;
    size_t buf_size;
    char** strs;
} cmpiutilStrArray;

/**
 * Create/initialize an cmpiutilStrArray object.
 * @return A pointer to the newly created/initialized cmpiutilStrArray object.
 *  This pointer will be used in all subsequent calls to cmpiutilStrArray_xxx
 *  functions and must eventually be passed to cmpiutilStrArray_destroy()
 */
cmpiutilStrArray* cmpiutilStrArray_create();

/**
 * Destroy an cmpiutilStrArray object and release all resources associated with it.
 * @param pra The pointer to the cmpiutilStrArray object to destroy.
 * @return 0 on success. Otherwise -1 on error.
 */
int cmpiutilStrArray_destroy(cmpiutilStrArray* pra);

/**
 * Append a string to the string array represented by a given cmpiutilStrArray
 * object.
 * @param pra A pointer to an cmpiutilStrArray object to append the string to.
 * @param str The string to append to the cmpiutilStrArray
 * @return A pointer to the cmpiutilStrArray on success or NULL on failure
 */
cmpiutilStrArray* cmpiutilStrArray_append(cmpiutilStrArray* pra, const char* str);

/**
 * Determine the number of strings contained by a given cmpiutilStrArray object.
 * @param pra A pointer to an cmpiutilStrArray object to get the number of 
 *  strings from.
 * @return The number of strings contained by the given cmpiutilStrArray object,
 *  or -1 on error.
 */
int cmpiutilStrArray_size(cmpiutilStrArray* pra);

/**
 * Reset/Clear a given cmpiutilStrArray object. After this call the cmpiutilStrArray
 * object will not contain any strings.
 * @param pra A pointer to the cmpiutilStrArray object to reset.
 * @return 0 on success. Otherwise -1.
 */
int cmpiutilStrArray_reset(cmpiutilStrArray* pra);

/**
 * Set an element within an cmpiutilStrArray object to a given string.
 * @param pra A pointer to the cmpiutilStrArray object to set the
 *  element in.
 * @param str The string to set the given element to.
 * @param ndx The array index of the element to set. This is zero relative,
 *  and must be less than the values returned by cmpiutilStrArray_size()
 * @return A pointer to the cmpiutilStrArray just modified on success, or
 *  NULL on failure.
 */
cmpiutilStrArray* cmpiutilStrArray_set_element(cmpiutilStrArray* pra, const char* str, size_t ndx);

/**
 * Get an element within an cmpiutilStrArray object.
 * @param pra A pointer to the cmpiutilStrArray object to get the string from.
 * @param ndx The index of the string to retrieve within the given cmpiutilStrArray.
 * @return A pointer to the string specified by ndx on success, or 
 *  NULL on failure.
 */
const char* cmpiutilStrArray_get_element(cmpiutilStrArray* pra, size_t ndx);

#ifdef __cplusplus
}
#endif

#endif	/* CMPIUTIL_STRING_H_GUARD__ */
